/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Génération d'une fiche prospect en pdf
 */

package com.ael.crm.mailing;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;
import java.util.Random;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.experlog.openeas.api.ServletRequest;
import org.experlog.openeas.api.Action;
import org.experlog.openeas.api.Session;

import com.ael.exception.ConfigurationException;
import com.ael.util.langage.EncodeurHtml;



public class GenMailing extends Action {

	class Destinataire {
		String id;
		String civilite;
		String nom;
		String prenom;
		String email;
		
		String getCiviliteComplete() {
			String civ = "";
			civ += (civilite.equals("") ? civilite : civilite + " ");
			civ += (prenom.equals("") ? prenom : prenom + " ");
			
			return (nom.equals("") ? "" : civ + nom);
		}
	}
	
	private interface Mailing {
		
		public void setText(Destinataire d)  throws MessagingException;
	}
	
	private class MailingHtml implements Mailing {
		
		private MimeBodyPart partie;
		private EncodeurHtml encodeur;
		private String corps;
		
		MailingHtml(Message courriel_, String corps_) throws MessagingException {
			encodeur = new EncodeurHtml();
			this.corps = encodeur.encoderEntites(encodeur.encoderBlanc(corps_));
			Multipart mp = new MimeMultipart();
			partie = new MimeBodyPart();
			mp.addBodyPart(partie);
			courriel_.setContent(mp);
		}
		
		public void setText(Destinataire d) throws MessagingException {
			partie.setText(
					corps.
						replaceAll("\\{user\\}", encodeur.encoderEntites(d.getCiviliteComplete())).
						replaceAll("\\{id\\}", d.id));
			partie.addHeaderLine("Content-Type: text/html; charset=\"iso-8859-15\"");
			//partie.addHeaderLine("Content-Transfer-Encoding: quoted-printable");
		}
		
	}
	
	private class MailingTexte implements Mailing {
		
		private Message courriel;
		private String corps;
		
		MailingTexte(Message courriel_, String corps_) {
			courriel = courriel_;
			corps = new EncodeurHtml().encoderBlanc(corps_);
		}
		
		public void setText(Destinataire d) throws MessagingException {
			courriel.setContent(corps.
						replaceAll("\\{user\\}", d.getCiviliteComplete()).
						replaceAll("\\{id\\}", d.id), "text/plain; charset=\"iso-8859-15\"");
		}
		
	}
	
	public boolean serverSide() { return true; }

	public boolean service(Session s, ServletRequest r, Object info) throws Exception {
	
		try {
			Connection conn = s.getConnection(null);
				
			String baseFact = s.getCookie().get("BaseDossier");
			String baseCrm = s.getCookie().get("CRMDossier");
			String idDossier = s.getCookie().get("Dossier_Id");
			String from = r.getParameter("from");
			String sujet = r.getParameter("sujet");
			String corps = r.getParameter("corps");
			String mode = r.getParameter("mode");
			String partenaire = r.getParameter("partenaire");
			String smtp = s.getAppConfiguration().get("email.MailHost");
				
			if (smtp == null || smtp.equals("")) {
				r.addParameter("ActionError", "Aucun serveur smtp disponible.");
				throw new ConfigurationException();
		  	}
				
			Properties prop = System.getProperties();
	        prop.put("mail.smtp.host", smtp);
	        javax.mail.Session session = javax.mail.Session.getDefaultInstance(prop,null);
	        Message courriel = new MimeMessage(session);
	            
	        // partie commune
			InternetAddress adresseFrom = new InternetAddress(from);
			courriel.setFrom(adresseFrom);
			courriel.setReplyTo(new InternetAddress[] {adresseFrom});
			courriel.setSubject(sujet);
			
			Mailing mailing = null;
			if (mode.equals("html")) // multipart
				mailing = new MailingHtml(courriel, corps);
			else if (mode.equals("texte"))
				mailing = new MailingTexte(courriel, corps);
				
			Statement stmt = conn.createStatement();
			String reqClient = "SELECT fc.Client_Id AS Id, c.Civ_Courte, i.Nom, i.Prenom, i.Email "+
			                   "FROM "+baseFact+".INTERVENANT i, "+baseFact+".FICHE_CLIENT fc, CIVILITE c "+ 
			                   "WHERE fc.Client_Id=i.Client_Id "+
			                   "AND fc.Mailing=1 AND i.Principal=1 AND i.Email!='' AND c.Civ_Id=i.Civilite";
			String reqProspect = "SELECT p.Id_Prospect AS Id, c.Civ_Courte, sp.Nom, sp.Prenom, sp.Email " + 
			                     "FROM "+baseCrm+".PROSPECT p, CIVILITE c " +
			                     "WHERE p.Mailing=1 AND p.Email!='' AND c.Civ_Id=p.Civilite";
			
		/*	String reqProspect = "SELECT sp.Id_Prospect AS Id, c.Civ_Courte, sp.Nom, sp.Prenom, sp.Email " + 
								 "FROM "+baseCrm+".SPAM_PROSPECT sp, CIVILITE c " +
								 "WHERE sp.Mailing=1 AND sp.Email!='' AND c.Civ_Id=sp.Civilite";
			String reqClient = "SELECT sfc.Client_Id AS Id, c.Civ_Courte, sfc.Nom, sfc.Prenom, sfc.Email " + 
			 				   "FROM "+baseCrm+".SPAM_FICHE_CLIENT sfc, CIVILITE c " +
			 				   "WHERE sfc.Mailing=1 AND sfc.Email!='' AND c.Civ_Id=sfc.Civilite"; */
			
			ResultSet rs;
			
			if (partenaire.equals("prospect"))
				rs = stmt.executeQuery(reqProspect);
			else if (partenaire.equals("client"))
				rs = stmt.executeQuery(reqClient);
			else // tous
				rs = stmt.executeQuery(reqClient + " UNION " + reqProspect);
			/*PreparedStatement psSupprRand = conn.prepareStatement(
												"DELETE FROM "+baseCrm+".MAILING WHERE Id_Partenaire=?");
			PreparedStatement psInsRand = conn.prepareStatement(
					"INSERT INTO "+baseCrm+".MAILING (Id_Mailing, Id_Partenaire) VALUES (?,?)");*/
			
			Vector<Destinataire> destinataires = new Vector<Destinataire>();
			while (rs.next()) {
				Destinataire d = new Destinataire();
				d.id = rs.getString("Id");
				d.civilite = rs.getString("Civ_Courte");
				d.nom = rs.getString("Nom");
				d.prenom = rs.getString("Prenom");
				d.email = rs.getString("Email");
				destinataires.add(d);
			}
			rs.close();
				
			Iterator<Destinataire> it = destinataires.iterator();
			//Random aleatoire = new Random();
			
			int i = 0;
			while (it.hasNext()) {
				i++;
				try {
					Destinataire d = it.next();
					courriel.setRecipient(Message.RecipientType.TO, new InternetAddress(d.email));
					// Paramétrer le corps
					mailing.setText(d);
					
					// partie commune
					//int rand = aleatoire.nextInt(1000000);
					Transport.send(courriel);
					/*try {
						psSupprRand.setString(1, d.id);
						psSupprRand.executeUpdate();
						psInsRand.setInt(1, rand);
						psInsRand.setString(2, d.id);
						psSupprRand.executeUpdate();
					} catch (SQLException se) {}*/
					if (i % 100 == 0)
						Thread.sleep(3000); // suspend toutes les 3 secondes
				} catch (MessagingException me) {
					me.printStackTrace();
					continue;
				}
			}
		} catch (ConfigurationException ce) {
		} catch (Exception e) {
			e.printStackTrace();
			r.addParameter("ActionError", "Erreur durant l'exécution du mailing");
		}
	
		return true;
	}
  
}
