/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/


package com.ael.crm.metier;

import java.util.Date;
import java.util.Vector;
import java.text.SimpleDateFormat;

/**
 * Une classe modélisant une communication
 * @author jm
 *
 */
public class Communication {
	
	private int idCommunication;
	private String canal;
	private String initiateur;
	private String createur;
	private String commentaire;
	private long dateCommunication = 0;
	private Vector<Resolution> resolutions = new Vector<Resolution>(); 

	/**
	 * Construit une <code>Communication</code>
	 * 
	 * @param	id_					identifiant entier de la communication
	 * @param 	initiateur_			initiateur de la communication
	 * @param	createur_			opérateur créateur de la communication
	 * @param	canal_				canal de communication
	 * @param	commentaire_rs		commentaires liés à la communication
	 * @param	dateCommunication_	date et heure de la communication en ms	
	 */
	public Communication (int id_,
			              String initiateur_,
			              String createur_,
			              String canal_,
			              String commentaire_,
			              long dateCommunication_) {
		idCommunication = id_;
		canal = canal_;
		createur = createur_;
		initiateur = initiateur_;
		commentaire = commentaire_;
		dateCommunication = dateCommunication_;
	}
	
	/**
	 * Construit une <code>Communication</code>
	 * 
	 * @param	id_					identifiant entier de la communication
	 */
	public Communication(int id_) {
		this(id_, "", "",  "", "", 0);
	}
	
	public void setIdCommunication(int id) { idCommunication = id; }
	public void setCanal(String canal_) { canal = canal_; }
	public void setCreateur(String createur_) { createur = createur_; }
	public void setInitiateur(String initiateur_) { initiateur = initiateur_; }
	public void setCommentaire(String commentaire_) { commentaire = commentaire_; }
	public void setDateCommunication(long dateCom) { dateCommunication = dateCom; }
	
	public int getIdCommunication() { return idCommunication; }
	public String getCanal() { return canal; }
	public String getCreateur() { return createur; }
	public String getInitiateur() { return initiateur; }
	public String getCommentaire() { return commentaire; }
	public long getDateCommunication() { return dateCommunication; }
	public Vector<Resolution> getResolutions() { return resolutions; }
	
	public String getDateCommunicationFormatee() {
		SimpleDateFormat formatDate = new SimpleDateFormat("dd-MM-yyyy HH:mm");
		return formatDate.format(new Date(dateCommunication));
	}
	
	public void ajouterResolution(Resolution r) {
		resolutions.add(r);
	}
	
	public void ajouterResolution(int idReso,
			                      String responsable,
			                      String statut,
			                      String priorite,
			                      String evenement,
			                      String description,
			                      long dateEvenement) {
		resolutions.add(
				new Resolution(
						idReso,
	                    responsable,
	                    statut,
	                    priorite,
	                    evenement,
	                    description,
	                    dateEvenement));
	}
	
}

